﻿[Version = "1.0.1"]
section Questionstar;

// shared authDomain = "http://localhost:81";
// shared apiDomain = "https://localhost:44303";
// shared repDomain = "https://localhost:44332";

// shared authDomain = "https://app.questionstar.ru";
// shared apiDomain = "https://external.questionstar.ru";
// shared repDomain = "https://reports.questionstar.ru";

shared authDomain = "https://app.questionstar.de";
shared apiDomain = "https://external.questionstar.de";
shared repDomain = "https://reports.questionstar.de";

shared qsAuthoriziationUrl = authDomain & "/confirm-app/microsoft-power-bi";
shared qsTokenUrl = apiDomain & "/api/v1/Account/AccessToken";
shared qsSurveysUrl = apiDomain & "/api/v1/Survey/Surveys/microsoftpowerbi";
shared qsDataUrl = repDomain & "/api/v1/ResultCaseDataTable/ResultCasesDataTables/DataJson";

shared biRedirectUrl = "https://oauth.powerbi.com/views/oauthredirect.html";
shared learnMoreUrl = "https://questionstar.com/powerbi";

clientId = "pbi.1234567890";
clientSecret = "85C981A2-125D-4689-A415-BD30BAA370F9";

RequestHeaders = [];

Questionstar = [
    Authentication = [
       OAuth = [
            StartLogin = StartLogin, 
            FinishLogin = FinishLogin,
            Label = Extension.LoadString("AuthenticationLabel")
        ]    
    ],
    Label = Extension.LoadString("DataSourceLabel")
];

StartLogin = (resourceUrl, state, display) =>
    let
        AuthorizeUrl = qsAuthoriziationUrl & "?" & Uri.BuildQueryString(
            [
                client_id = clientId,
                response_type = "code",
                state = state,
                redirect_uri = biRedirectUrl
            ])
     in
         [
             LoginUri = AuthorizeUrl,
             CallbackUri = biRedirectUrl,
             WindowHeight = 800,
             WindowWidth = 600,
             Context = null
         ];

FinishLogin = (context, callbackUri, state) =>
    let
        Parts = Uri.Parts(callbackUri)[Query]
    in
        TokenMethod(Parts[code]);

TokenMethod = (code) =>
    let
        Response = Web.Contents(qsTokenUrl,[
            Content = Text.ToBinary( Uri.BuildQueryString([
                client_id = clientId,
                client_secret = clientSecret,
                code = code,
                grant_type = "authorization_code",
                redirect_uri = biRedirectUrl])),
            Headers = [
                #"Content-type" = "application/x-www-form-urlencoded",
                #"Accept" = "application/json"]]),
            Parts = Json.Document(Response) 
        in
            Parts;
     
Table.ToNavigationTable = (
    table as table, 
    keyColumns as list, 
    nameColumn as text,
    dataColumn as text,
    itemKindColumn as text,
    itemNameColumn as text,
    isLeafColumn as text
    ) as table =>
        let
            tableType = Value.Type(table),
            newTableType = Type.AddTableKey(tableType, keyColumns, true)
                meta
                [
                    NavigationTable.NameColumn = nameColumn,
                    NavigationTable.DataColumn = dataColumn,
                    NavigationTable.ItemKindColumn = itemKindColumn, 
                    Preview.DelayColumn = itemNameColumn, 
                    NavigationTable.IsLeafColumn = isLeafColumn
                ],
            navigationTable = Value.ReplaceType(table, newTableType)
        in
            navigationTable;

SurveysNavTable = (url as text) as table =>
    let
        Source = Json.Document(Web.Contents(qsSurveysUrl)),
        #"Converted to Table" = Table.FromRecords({Source}),
        #"Expanded data" = Table.ExpandListColumn(#"Converted to Table", "data"),
        #"Expanded data1" = Table.ExpandRecordColumn(#"Expanded data", "data", { "id", "name" }, { "data.id", "data.name" }), 
        #"Changed Type" = Table.TransformColumnTypes(#"Expanded data1", {{ "data.id", type text }, { "data.name", type text }}),
        #"Removed Columns" = Table.RemoveColumns(#"Changed Type", { "success", "errors" }),
        withData = Table.AddColumn( #"Removed Columns", "Data", each Questionstar.Feed(Record.Field(_, "data.id"))), // GetQuestions(Record.Field(_, "data.id"))
        withItemKind = Table.AddColumn(withData, "ItemKind", each "Table"),
        withItemName = Table.AddColumn(withItemKind, "ItemName", each "Table"),
        withItemIsLeaf = Table.AddColumn(withItemName, "IsLeaf", each true),
        #"Renamed Columns" = Table.RenameColumns(withItemIsLeaf, {{ "data.id", "Id" }, { "data.name", "Name" }}),
        navTable = Table.ToNavigationTable(#"Renamed Columns", {"Id"}, "Name", "Data", "ItemKind", "ItemName", "IsLeaf")
    in
        navTable;

Questionstar.Feed =(id as text) =>
    let
        source = Web.Contents(qsDataUrl & "/" & id), 
            json = Json.Document(source),
            data = json[data],
            asTable = Table.FromList(data, Splitter.SplitByNothing()),
            CheckEmpty = if Table.IsEmpty(asTable) then #table(type table[Column1 = [EventItemId = any, EventItemDescription = any]],{}) else asTable,
            fields = Record.FieldNames(Table.FirstValue(CheckEmpty, [Empty = null])),
            expandAll = Table.ExpandRecordColumn(CheckEmpty, "Column1", fields)
        in
            expandAll;

[DataSource.Kind = "Questionstar", Publish = "Questionstar.Publish"]
shared Questionstar.Contents = () => SurveysNavTable(qsSurveysUrl) as table;

Questionstar.Publish = [
    Beta = true,
    Category = "Other",
    ButtonText = {
        Extension.LoadString("ButtonTitle"),
        Extension.LoadString("ButtonHelp")
    },
    LearnMoreUrl = learnMoreUrl,
    SourceImage = Questionstar.Icons,
    SourceTypeImage = Questionstar.Icons
];
Questionstar.Icons = [
    Icon16 = {
        Extension.Contents("Questionstar16.png"),
        Extension.Contents("Questionstar20.png"),
        Extension.Contents("Questionstar24.png"),
        Extension.Contents("Questionstar32.png")
    },
    Icon32 = {
        Extension.Contents("Questionstar32.png"),
        Extension.Contents("Questionstar40.png"),
        Extension.Contents("Questionstar48.png"),
        Extension.Contents("Questionstar64.png")
    }
];